@@ -45,14 +45,12 @@ from splitgraph.ingestion.singer.commandline import singer_group
 
 logger = logging.getLogger()
 
-# Patch click_log's handler to restore its behaviour where INFO level goes to stdout.
-
 
 class ClickHandler(logging.Handler):
     def emit(self, record):
         try:
             msg = self.format(record)
-            click.echo(msg, err=record.levelno != logging.INFO)
+            click.echo(msg, err=True)
         except Exception:
             self.handleError(record)
 
@@ -78,11 +76,11 @@ def _patch_wrap_text():
 
 def _do_version_check():
     """Do a pre-flight version check -- by default we only do it once a day"""
-    from splitgraph.cloud import AuthAPIClient
+    from splitgraph.cloud import RESTAPIClient
     from packaging.version import Version
-    from splitgraph.config import CONFIG
+    from splitgraph.config import CONFIG, get_singleton
 
-    api_client = AuthAPIClient(CONFIG["SG_UPDATE_REMOTE"])
+    api_client = RESTAPIClient(get_singleton(CONFIG, "SG_UPDATE_REMOTE"))
     latest = api_client.get_latest_version()
 
     if not latest:
@@ -104,6 +102,12 @@ class WithExceptionHandler(click.Group):
 
     def invoke(self, ctx):
         from splitgraph.engine import get_engine
+        import psycopg2.extensions
+        import psycopg2.extras
+
+        # Allow users to send SIGINT to quickly terminate sgr (instead of waiting for a PG
+        # statement to finish)
+        psycopg2.extensions.set_wait_callback(psycopg2.extras.wait_select)
 
         engine = get_engine()
         try:
