@@ -1,14 +1,20 @@
 """Command line routines related to registering/setting up connections to the Splitgraph registry."""
+import hashlib
 import logging
+import os
 import shutil
+import string
 import subprocess
 from copy import copy
-from typing import Dict, Optional
+from glob import glob
+from typing import Dict, Optional, cast, Tuple
 from urllib.parse import urlparse, quote
 
 import click
 from click import wrap_text
+from tqdm import tqdm
 
+from splitgraph.cloud.models import Metadata, RepositoriesYAML, AddExternalRepositoryRequest
 from splitgraph.commandline.common import (
     ImageType,
     RepositoryType,
@@ -16,17 +22,24 @@ from splitgraph.commandline.common import (
     Color,
 )
 from splitgraph.commandline.engine import patch_and_save_config, inject_config_into_engines
+from splitgraph.core.output import pluralise
 
 # Hardcoded database name for the Splitgraph DDN (ddn instead of sgregistry)
+from splitgraph.config.config import get_from_subsection
+
 _DDN_DBNAME = "ddn"
 
+VALID_FILENAME_CHARACTERS = string.ascii_letters + string.digits + "-_."
+
 
 @click.command("register")
 @click.option("--username", prompt=True)
 @click.password_option()
 @click.option("--email", prompt=True)
 @click.option(
-    "--remote", default="data.splitgraph.com", help="Name of the remote registry to register on.",
+    "--remote",
+    default="data.splitgraph.com",
+    help="Name of the remote registry to register on.",
 )
 @click.option("--accept-tos", is_flag=True, help="Accept the registry's Terms of Service")
 @click.option(
@@ -40,11 +53,11 @@ def register_c(username, password, email, remote, accept_tos, skip_inject):
     obtains a set of machine (API) credentials for the client to communicate
     with the registry and configures the data.splitgraph.com engine.
     """
-    from splitgraph.cloud import AuthAPIClient, DEFAULT_REMOTES
+    from splitgraph.cloud import RESTAPIClient, DEFAULT_REMOTES
     from splitgraph.config import CONFIG
     from splitgraph.config.config import get_all_in_subsection
 
-    client = AuthAPIClient(remote)
+    client = RESTAPIClient(remote)
     tos = client.tos()
 
     if tos and not accept_tos:
@@ -52,10 +65,12 @@ def register_c(username, password, email, remote, accept_tos, skip_inject):
         click.confirm("Do you accept the Terms of Service?", default=False, abort=True)
     click.echo("Registering the user...")
 
-    uuid, access, refresh = client.register(username, password, email, accept_tos=True)
+    uuid, access, refresh = cast(
+        Tuple[str, str, str], client.register(username, password, email, accept_tos=True)
+    )
     click.echo("Registration successful. UUID %s" % uuid)
 
-    key, secret = client.create_machine_credentials(access, password)
+    key, secret = cast(Tuple[str, str], client.create_machine_credentials(access, password))
     click.echo("Acquired refresh token and API keys")
 
     repo_lookup = _update_repo_lookup(CONFIG, remote)
@@ -126,7 +141,9 @@ def _construct_search_url(gql_endpoint: str, query: str):
 @click.option("--username", prompt="Username or e-mail")
 @click.option("--password", default=None)
 @click.option(
-    "--remote", default="data.splitgraph.com", help="Name of the remote registry to log into.",
+    "--remote",
+    default="data.splitgraph.com",
+    help="Name of the remote registry to log into.",
 )
 @click.option(
     "--overwrite", is_flag=True, help="Overwrite old API keys in the config if they exist"
@@ -149,9 +166,9 @@ def login_c(username, password, remote, overwrite, skip_inject):
     """
     from splitgraph.config import CONFIG
     from splitgraph.config.config import get_all_in_subsection
-    from splitgraph.cloud import AuthAPIClient, get_token_claim, DEFAULT_REMOTES
+    from splitgraph.cloud import RESTAPIClient, get_token_claim, DEFAULT_REMOTES
 
-    client = AuthAPIClient(remote)
+    client = RESTAPIClient(remote)
 
     if not password:
         profile_url = _construct_user_profile_url(client.endpoint)
@@ -161,7 +178,7 @@ def login_c(username, password, remote, overwrite, skip_inject):
             hide_input=True,
         )
 
-    access, refresh = client.get_refresh_token(username, password)
+    access, refresh = cast(Tuple[str, str], client.get_refresh_token(username, password))
 
     # Extract namespace from the access token since we might have logged in with an e-mail.
     namespace = get_token_claim(access, "username")
@@ -186,7 +203,9 @@ def login_c(username, password, remote, overwrite, skip_inject):
 
     # Get new tokens in any case if we're logging in under a different username.
     try:
-        username_changed = namespace != CONFIG["remotes"][remote]["SG_NAMESPACE"]
+        username_changed = namespace != get_from_subsection(
+            CONFIG, "remotes", remote, "SG_NAMESPACE"
+        )
     except KeyError:
         username_changed = False
 
@@ -196,7 +215,7 @@ def login_c(username, password, remote, overwrite, skip_inject):
         or overwrite
         or username_changed
     ):
-        key, secret = client.create_machine_credentials(access, password)
+        key, secret = cast(Tuple[str, str], client.create_machine_credentials(access, password))
         config_patch["remotes"][remote]["SG_ENGINE_USER"] = key
         config_patch["remotes"][remote]["SG_ENGINE_PWD"] = secret
         click.echo("Acquired new API keys")
@@ -211,7 +230,9 @@ def login_c(username, password, remote, overwrite, skip_inject):
 @click.option("--api-key", prompt="API key")
 @click.option("--api-secret", prompt="API secret", confirmation_prompt=False, hide_input=True)
 @click.option(
-    "--remote", default="data.splitgraph.com", help="Name of the remote registry to log into.",
+    "--remote",
+    default="data.splitgraph.com",
+    help="Name of the remote registry to log into.",
 )
 @click.option(
     "-s", "--skip-inject", is_flag=True, help="Don't try to copy the config into all engines"
@@ -222,12 +243,12 @@ def login_api_c(api_key, api_secret, remote, skip_inject):
     This will inject the API keys for the registry into the configuration file
     and generate a new access token.
     """
-    from splitgraph.cloud import AuthAPIClient, get_token_claim, DEFAULT_REMOTES
+    from splitgraph.cloud import RESTAPIClient, get_token_claim, DEFAULT_REMOTES
     from splitgraph.config import CONFIG
     from splitgraph.config.config import get_all_in_subsection
 
-    client = AuthAPIClient(remote)
-    access = client.get_access_token_from_api(api_key, api_secret)
+    client = RESTAPIClient(remote)
+    access = cast(str, client.get_access_token_from_api(api_key, api_secret))
 
     namespace = get_token_claim(access, "username")
 
@@ -295,13 +316,13 @@ def curl_c(remote, request_type, image, request_params, curl_args):
     with `--curl-args`, e.g. `--curl-args --cacert --curl-args /path/to/ca.pem`.
     """
     from splitgraph.config import CONFIG
-    from splitgraph.cloud import AuthAPIClient, get_headers
+    from splitgraph.cloud import RESTAPIClient, get_headers
 
     repository, hash_or_tag = image
 
     # Craft a request
-    config = CONFIG["remotes"][remote]
-    access_token = AuthAPIClient(remote).access_token
+    config = cast(Dict[str, str], CONFIG["remotes"][remote])
+    access_token = RESTAPIClient(remote).access_token
     headers = get_headers()
     headers.update({"Authorization": "Bearer " + access_token})
 
@@ -387,7 +408,7 @@ def sql_c(remote, show_all, json, query):
 @click.argument("repository", type=RepositoryType(exists=False))
 @click.argument("readme", type=click.File("r"))
 def readme_c(remote, repository, readme):
-    """Upload or a README to a Splitgraph repository.
+    """Upload a README to a Splitgraph repository.
 
     The repository must have already been pushed. The README must be a file in Markdown format.
     """
@@ -461,26 +482,30 @@ def metadata_c(ctx, remote, repository, metadata_file):
     import yaml
     from splitgraph.cloud import GQLAPIClient
 
-    metadata = yaml.safe_load(metadata_file)
+    metadata = Metadata.parse_obj(yaml.safe_load(metadata_file))
+    metadata = _prepare_metadata(metadata)
 
+    client = GQLAPIClient(remote)
+    client.upsert_metadata(repository.namespace, repository.repository, metadata)
+    click.echo("Metadata updated for repository %s." % str(repository))
+
+
+def _prepare_metadata(metadata, readme_basedir="."):
     keys = ["readme", "description", "topics", "sources", "license", "extra_metadata"]
-    if all(k not in metadata for k in keys):
+    if all(metadata.__getattribute__(k) is None for k in keys):
         raise click.UsageError(
             "Invalid metadata file. File must contain at least one of " f"{'/'.join(keys)} keys."
         )
+    readme_file = None
+    if isinstance(metadata.readme, str):
+        readme_file = metadata.readme
+    elif isinstance(metadata.readme, Metadata.Readme) and metadata.readme.file:
+        readme_file = metadata.readme.file
+    if readme_file:
+        with open(os.path.join(readme_basedir, readme_file), "r") as f:
+            metadata.readme = Metadata.Readme(text=f.read())
 
-    kwargs = {}
-    if "readme" in metadata:
-        with open(metadata["readme"], "r") as f:
-            kwargs["readme"] = f.read()
-
-    for k in keys:
-        if k != "readme" and k in metadata:
-            kwargs[k] = metadata[k]
-
-    client = GQLAPIClient(remote)
-    client.upsert_metadata(repository.namespace, repository.repository, **kwargs)
-    click.echo("Metadata updated for repository %s." % str(repository))
+    return metadata
 
 
 @click.command("search")
@@ -526,6 +551,277 @@ def search_c(remote, query, limit=10):
         )
 
 
+def _normalise_filename(filename):
+    """
+    Strips out odd characters from a string so that it can be used as a valid filename.
+    In order for the result to be unique, we also makea short hash of the original
+    string and stick it on the end
+    """
+    base = "".join(c for c in filename if c in VALID_FILENAME_CHARACTERS)
+    short_hash = hashlib.sha1(filename.encode()).hexdigest()[:4]
+    return f"{base}.{short_hash}"
+
+
+@click.command("dump")
+@click.option("--remote", default="data.splitgraph.com", help="Name of the remote registry to use.")
+@click.option("--readme-dir", default="./readmes", help="Directory to dump the data into")
+@click.option("--repositories-file", "-f", default="repositories.yml", type=click.File("w"))
+@click.argument("limit_repositories", type=str, nargs=-1)
+def dump_c(remote, readme_dir, repositories_file, limit_repositories):
+    """
+    Dump a Splitgraph catalog to a YAML file.
+
+    This creates a repositories.yml file in the target directory as well as a subdirectory `readmes`
+    with all the repository readmes. This file can be used to recreate all catalog metadata
+    and all external data source settings for a repository using `sgr cloud load`.
+    """
+    import yaml
+    from splitgraph.cloud import GQLAPIClient
+
+    client = GQLAPIClient(remote)
+    repositories = client.load_all_repositories(limit_to=limit_repositories)
+
+    _dump_readmes_to_dir(repositories, readme_dir)
+
+    yaml.dump(
+        {"repositories": [r.dict(by_alias=True, exclude_unset=True) for r in repositories]},
+        repositories_file,
+        sort_keys=False,
+    )
+
+
+@click.command("load")
+@click.option("--remote", default="data.splitgraph.com", help="Name of the remote registry to use.")
+@click.option(
+    "--readme-dir", default="./readmes", help="Path to the directory with the README files"
+)
+@click.option("--repositories-file", "-f", default="repositories.yml", type=click.File("r"))
+@click.argument("limit_repositories", type=str, nargs=-1)
+def load_c(remote, readme_dir, repositories_file, limit_repositories):
+    """
+    Load a Splitgraph catalog from a YAML file.
+
+    This will load a repositories.yml file and the `readmes` subdirectory produced by
+    `sgr cloud dump` back into a remote Splitgraph catalog.
+
+    The format is an extension of the format accepted by `sgr cloud metadata` to include multiple
+    repositories. README files are read from the `readmes` subdirectory.
+
+    \b
+    ```
+    credentials:      # Optional credentials to access remote data sources
+      my_bucket:
+        plugin: csv
+        data:
+          s3_access_key: ...
+          s3_secret_key: ...
+    repositories:
+    - namespace: my_username
+      repository: repository
+      metadata:
+        readme: dataset-readme.md
+        description: Dataset description (160 characters max).
+        topics:
+          - topic_1
+          - topic_2
+        sources:
+          - anchor: Source
+            href: https://www.splitgraph.com
+            isCreator: true
+            isSameAs: false
+          - anchor: Source 2
+            href: https://www.splitgraph.com
+            isCreator: false
+            isSameAs: true
+        license: Public Domain
+        extra_metadata:
+          key_1:
+            key_1_1: value_1_1
+            key_1_2: value_1_2
+          key_2:
+            key_2_1: value_2_1
+            key_2_2: value_2_2
+      external:
+        credential: my_bucket
+        plugin: csv
+        params:
+          s3_bucket: my_bucket
+        tables:
+          table_1:
+            schema:
+            - name: column_1
+              type: text
+            - name: column_2
+              type: integer
+            options:
+              s3_object: some/s3_key.csv
+    ```
+    """
+    import yaml
+    from splitgraph.cloud import GQLAPIClient
+    from splitgraph.cloud import RESTAPIClient
+
+    repo_yaml = RepositoriesYAML.parse_obj(yaml.safe_load(repositories_file))
+
+    # Set up and load credential IDs from the remote to allow users to refer to them by ID
+    # or by a name.
+    rest_client = RESTAPIClient(remote)
+    gql_client = GQLAPIClient(remote)
+    credential_map = _build_credential_map(rest_client, credentials=repo_yaml.credentials or {})
+
+    repositories = repo_yaml.repositories
+
+    if limit_repositories:
+        repositories = [
+            r for r in repositories if f"{r.namespace}/{r.repository}" in limit_repositories
+        ]
+
+    logging.info("Uploading images...")
+    external_repositories = []
+    for repository in repositories:
+        if repository.external:
+            external_repository = AddExternalRepositoryRequest.from_external(
+                repository.namespace, repository.repository, repository.external, credential_map
+            )
+            external_repositories.append(external_repository)
+    rest_client.bulk_upsert_external(repositories=external_repositories)
+    logging.info(f"Uploaded images for {pluralise('repository', len(external_repositories))}")
+
+    logging.info("Updating metadata...")
+    namespace_list = []
+    repository_list = []
+    metadata_list = []
+    for repository in repositories:
+        if repository.metadata:
+            namespace_list.append(repository.namespace)
+            repository_list.append(repository.repository)
+
+            metadata = _prepare_metadata(repository.metadata, readme_basedir=readme_dir)
+            metadata_list.append(metadata)
+    gql_client.bulk_upsert_metadata(namespace_list, repository_list, metadata_list)
+    logging.info(f"Updated metadata for {pluralise('repository', len(repository_list))}")
+
+
+def _build_credential_map(auth_client, credentials=None):
+    credential_map = {}
+    if credentials:
+        logging.info("Setting up credentials on the remote...")
+        for credential_name, credential in credentials.items():
+            credential_id = auth_client.ensure_external_credential(
+                credential_data=credential.data,
+                credential_name=credential_name,
+                plugin_name=credential.plugin,
+            )
+            logging.info("%s: %s", credential_name, credential_id)
+            credential_map[credential_name] = credential_id
+
+    # Load any other credentials too (so that we can use existing credentials by
+    # name in YAML files without redefining them)
+    response = auth_client.list_external_credentials()
+    for credential in response.credentials:
+        if credential.credential_name not in credential_map:
+            credential_map[credential.credential_name] = credential.credential_id
+
+    return credential_map
+
+
+def _dump_readmes_to_dir(repositories, readme_dir):
+    """
+    READMEs aren't rendering very nicely in YAML so we instead write them out to
+    individual files and replace the README contents with their file paths.
+
+    :param repositories: will be modified by this function
+    """
+    try:
+        os.mkdir(readme_dir)
+    except FileExistsError:
+        logging.info("Directory %s already exists, cleaning out", readme_dir)
+        for path in glob(os.path.join(readme_dir, "*.md")):
+            os.unlink(path)
+    for repo in repositories:
+        if not repo.metadata:
+            continue
+
+        readme_text = None
+        if isinstance(repo.metadata.readme, str):
+            readme_text = repo.metadata.readme
+        if (
+            isinstance(repo.metadata.readme, Metadata.Readme)
+            and repo.metadata.readme.file is None
+            and repo.metadata.readme.text is not None
+        ):
+            readme_text = repo.metadata.readme.text
+
+        if readme_text:
+            filename = f"{repo.namespace}-{repo.repository}"
+            filename = _normalise_filename(filename)
+            filename = f"{filename}.md"
+            logging.info("Writing %s", filename)
+            path = os.path.join(readme_dir, filename)
+            with open(path, "w") as f:
+                f.write(readme_text)
+            repo.metadata.readme = Metadata.Readme(file=filename)
+
+
+@click.command("token")
+@click.option("--remote", default="data.splitgraph.com", help="Name of the remote registry to use.")
+def token_c(remote):
+    """Output an up-to-date access token for a remote."""
+    from splitgraph.cloud import RESTAPIClient
+
+    client = RESTAPIClient(remote)
+    click.echo(client.access_token)
+
+
+@click.command("add")
+@click.option("--remote", help="Name for the remote (infer from the domain name by default)")
+@click.option(
+    "-s", "--skip-inject", is_flag=True, help="Don't try to copy the config into all engines"
+)
+@click.argument("domain_name")
+def add_c(remote, skip_inject, domain_name):
+    """Add a remote Splitgraph registry to .sgconfig with default parameters"""
+    from splitgraph.config.config import get_all_in_subsection
+    from splitgraph.config import CONFIG
+
+    if domain_name.startswith(("data", "www", "api")):
+        raise click.BadArgumentUsage(
+            "Use the base domain name, e.g. " "splitgraph.com instead of data.splitgraph.com"
+        )
+
+    if not remote:
+        remote = domain_name.split(".")[0]
+
+    remote_params_patch: Dict[str, str] = {
+        "SG_IS_REGISTRY": "true",
+        "SG_ENGINE_HOST": f"data.{domain_name}",
+        "SG_ENGINE_PORT": "5432",
+        "SG_ENGINE_DB_NAME": "sgregistry",
+        "SG_AUTH_API": f"https://api.{domain_name}/auth",
+        "SG_QUERY_API": f"https://data.{domain_name}",
+        "SG_GQL_API": f"https://api.{domain_name}/gql/cloud/graphql",
+    }
+
+    click.echo("Adding remote %s to the config. Parameters: %s" % (remote, remote_params_patch))
+
+    remote_params = get_all_in_subsection(CONFIG, "remotes", remote) or {}
+    remote_params.update(remote_params_patch)
+
+    config_patch = {
+        "SG_REPO_LOOKUP": _update_repo_lookup(CONFIG, remote),
+        "remotes": {remote: remote_params},
+    }
+    config_path = patch_and_save_config(CONFIG, config_patch)
+
+    if not skip_inject:
+        inject_config_into_engines(CONFIG["SG_ENGINE_PREFIX"], config_path)
+
+    click.echo(
+        "Done. You can now register or "
+        "log in to the instance with sgr cloud register/login(-api)."
+    )
+
+
 @click.group("cloud")
 def cloud_c():
     """Manage connections to Splitgraph Cloud."""
@@ -540,3 +836,7 @@ cloud_c.add_command(readme_c)
 cloud_c.add_command(description_c)
 cloud_c.add_command(metadata_c)
 cloud_c.add_command(search_c)
+cloud_c.add_command(dump_c)
+cloud_c.add_command(load_c)
+cloud_c.add_command(token_c)
+cloud_c.add_command(add_c)
