@@ -1,66 +1,21 @@
 import logging
 import os
 import platform
-import time
-from io import BytesIO
 from pathlib import Path, PureWindowsPath
-from tarfile import TarFile, TarInfo
-from typing import Dict, TYPE_CHECKING
+from typing import Dict, Any, cast, Optional
 from urllib.parse import urlparse
 
 import click
 from tqdm import tqdm
 
 from splitgraph.__version__ import __version__
-from splitgraph.config import CONFIG, SG_CMD_ASCII
+from splitgraph.config import CONFIG, SG_CMD_ASCII, get_singleton
 from splitgraph.exceptions import DockerUnavailableError, EngineSetupError
-
-if TYPE_CHECKING:
-    from docker.models.containers import Container
-
+from splitgraph.utils.docker import get_docker_client, copy_to_container
 
 DEFAULT_ENGINE = "default"
 
 
-def get_docker_client():
-    """Wrapper around client.from_env() that also pings the daemon
-    to make sure it can connect and if not, raises an error."""
-    import docker
-
-    try:
-        client = docker.from_env()
-        client.ping()
-        return client
-    except Exception as e:
-        raise DockerUnavailableError("Could not connect to the Docker daemon") from e
-
-
-def copy_to_container(container: "Container", source_path: str, target_path: str) -> None:
-    """
-    Copy a file into a Docker container
-
-    :param container: Container object
-    :param source_path: Source file path
-    :param target_path: Target file path (in the container)
-    :return:
-    """
-    # https://github.com/docker/docker-py/issues/1771
-    with open(source_path, "rb") as f:
-        data = f.read()
-
-    tarinfo = TarInfo(name=os.path.basename(target_path))
-    tarinfo.size = len(data)
-    tarinfo.mtime = int(time.time())
-
-    stream = BytesIO()
-    tar = TarFile(fileobj=stream, mode="w")
-    tar.addfile(tarinfo, BytesIO(data))
-    tar.close()
-
-    stream.seek(0)
-    container.put_archive(path=os.path.dirname(target_path), data=stream.read())
-
-
 def patch_and_save_config(config, patch):
     from splitgraph.config.config import patch_config
     from splitgraph.config.system_config import HOME_SUB_DIR
@@ -211,8 +166,8 @@ def _pretty_pull(client, image):
             position=1,
             ascii=use_ascii,
         ) as extract_bar:
-            download_progress = {}
-            extract_progress = {}
+            download_progress: Dict[str, Dict[str, Any]] = {}
+            extract_progress: Dict[str, Dict[str, Any]] = {}
             for progress in client.api.pull(image, stream=True, decode=True):
                 progress_text = "%s: %s" % (progress.get("id", ""), progress.get("status", ""))
                 if progress["status"] == "Downloading":
@@ -221,7 +176,12 @@ def _pretty_pull(client, image):
                     _update_bar(progress, progress_text, extract_bar, extract_progress)
 
 
-def _update_bar(progress, progress_text, bar, progress_data):
+def _update_bar(
+    progress: Dict[str, Any],
+    progress_text: str,
+    bar: tqdm,
+    progress_data: Dict[str, Dict[str, Any]],
+):
     if progress.get("progressDetail"):
         progress_data[progress["id"]] = progress["progressDetail"]
 
@@ -237,7 +197,7 @@ def _update_bar(progress, progress_text, bar, progress_data):
     default="splitgraph/engine:%s" % __version__,
     help="Docker image with the Splitgraph engine",
 )
-@click.option("-p", "--port", type=int, default=5432, help="Port to start the engine on")
+@click.option("-p", "--port", type=int, default=6432, help="Port to start the engine on")
 @click.option("-u", "--username", default="sgr")
 @click.option("--no-init", default=False, help="Don't run `sgr init` on the engine", is_flag=True)
 @click.option(
@@ -360,6 +320,7 @@ def add_engine_c(
         "SG_ENGINE_ADMIN_PWD": password,
     }
 
+    config_patch: Dict[str, Any]
     if not no_sgconfig:
         if name != DEFAULT_ENGINE and not set_default:
             config_patch = {"remotes": {name: conn_params}}
@@ -371,7 +332,9 @@ def add_engine_c(
         config_path = CONFIG["SG_CONFIG_FILE"]
 
     if not no_init:
-        engine = PostgresEngine(name=name, conn_params=conn_params)
+        engine = PostgresEngine(
+            name=name, conn_params=cast(Optional[Dict[str, Optional[str]]], conn_params)
+        )
         engine.initialize()
         engine.commit()
         click.echo("Engine initialized successfully.")
@@ -487,7 +450,7 @@ def configure_engine_c(name):
     container_name = _get_container_name(name)
     container = client.containers.get(container_name)
 
-    copy_to_container(container, CONFIG["SG_CONFIG_FILE"], "/.sgconfig")
+    copy_to_container(container, get_singleton(CONFIG, "SG_CONFIG_FILE"), "/.sgconfig")
     logging.info("Config updated for container %s", container.name)
 
 
