@@ -1,5 +1,20 @@
 from abc import ABCMeta, abstractmethod
-from typing import Dict, Tuple, Any, NamedTuple, Optional, List, Sequence, Union, TypeVar
+from typing import (
+    Dict,
+    Tuple,
+    Any,
+    NamedTuple,
+    Optional,
+    List,
+    Sequence,
+    Union,
+    TypeVar,
+    TYPE_CHECKING,
+    NewType,
+)
+
+if TYPE_CHECKING:
+    from splitgraph.cloud.models import ExternalTableRequest
 
 Changeset = Dict[Tuple[str, ...], Tuple[bool, Dict[str, Any], Dict[str, Any]]]
 
@@ -19,11 +34,11 @@ SourcesList = List[Dict[str, str]]
 ProvenanceLine = Dict[str, Union[str, List[str], List[bool], SourcesList]]
 
 # Ingestion-related params
-Credentials = Dict[str, Any]
-Params = Dict[str, Any]
-TableParams = Dict[str, Any]
+Credentials = NewType("Credentials", Dict[str, Any])
+Params = NewType("Params", Dict[str, Any])
+TableParams = NewType("TableParams", Dict[str, Any])
 TableInfo = Union[List[str], Dict[str, Tuple[TableSchema, TableParams]]]
-SyncState = Dict[str, Any]
+SyncState = NewType("SyncState", Dict[str, Any])
 
 
 class MountError(NamedTuple):
@@ -32,8 +47,10 @@ class MountError(NamedTuple):
     error_text: str
 
 
-PreviewResult = Dict[str, Union[MountError, List[Dict[str, Any]]]]
-IntrospectionResult = Dict[str, Union[Tuple[TableSchema, TableParams], MountError]]
+PreviewResult = NewType("PreviewResult", Dict[str, Union[MountError, List[Dict[str, Any]]]])
+IntrospectionResult = NewType(
+    "IntrospectionResult", Dict[str, Union[Tuple[TableSchema, TableParams], MountError]]
+)
 
 T = TypeVar("T")
 
@@ -51,6 +68,12 @@ def unwrap(
     return good, bad
 
 
+def get_table_params(table_info: TableInfo, table_name: str) -> TableParams:
+    if isinstance(table_info, dict) and table_name in table_info:
+        return table_info[table_name][1]
+    return TableParams({})
+
+
 class Comparable(metaclass=ABCMeta):
     @abstractmethod
     def __lt__(self, other: Any) -> bool:
@@ -58,15 +81,15 @@ class Comparable(metaclass=ABCMeta):
 
 
 def dict_to_table_schema_params(
-    tables: Dict[str, Dict[str, Any]]
+    tables: Dict[str, "ExternalTableRequest"]
 ) -> Dict[str, Tuple[TableSchema, TableParams]]:
     return {
         t: (
             [
                 TableColumn(i + 1, cname, ctype, False, None)
-                for (i, (cname, ctype)) in enumerate(tsp["schema"].items())
+                for (i, (cname, ctype)) in enumerate((tsp.schema_ or {}).items())
             ],
-            tsp.get("options", {}),
+            TableParams(tsp.options),
         )
         for t, tsp in tables.items()
     }
@@ -74,11 +97,11 @@ def dict_to_table_schema_params(
 
 def table_schema_params_to_dict(
     tables: Dict[str, Tuple[TableSchema, TableParams]]
-) -> Dict[str, Dict[str, Dict[str, str]]]:
+) -> Dict[str, Dict[str, Dict[str, Any]]]:
     return {
         t: {
             "schema": {c.name: c.pg_type for c in ts},
-            "options": {tpk: str(tpv) for tpk, tpv in tp.items()},
+            "options": tp,
         }
         for t, (ts, tp) in tables.items()
     }
