@@ -5,12 +5,13 @@ import re
 from abc import ABC
 from contextlib import contextmanager
 from random import getrandbits
-from typing import Optional, Dict, List, Tuple
+from typing import Optional, Dict, List, Tuple, Generator
 
 import docker.errors
 import pydantic
 from docker import DockerClient
 from docker.models.containers import Container
+from docker.types import LogConfig
 
 from splitgraph.core.repository import Repository
 from splitgraph.core.types import (
@@ -48,9 +49,19 @@ from .utils import (
     select_streams,
     get_pk_cursor_fields,
 )
+from ..dbt.utils import run_dbt_transformation_from_git
 from ..singer.common import store_ingestion_state, add_timestamp_tags
 
 
+@contextmanager
+def delete_schema_at_end(engine: "PostgresEngine", schema: str) -> Generator:
+    try:
+        yield
+    finally:
+        engine.delete_schema(schema)
+        engine.commit()
+
+
 class AirbyteDataSource(SyncableDataSource, ABC):
     """Generic data source for Airbyte-compliant sources.
     We run ingestion by combining an Airbyte source and the Airbyte Postgres destination.
@@ -82,8 +93,58 @@ class AirbyteDataSource(SyncableDataSource, ABC):
         },
     }
 
+    # Base parameters/credentials for this data source (allowing users to pass in a Git repo
+    # with a dbt project). Note that classes inheriting this should keep this JSONSchema by
+    # redefining it as:
+    #
+    #   credentials_schema = merge_jsonschema(
+    #       AirbyteDataSource.credentials_schema,
+    #       {"type": "object", "properties": {...}},
+    #   )
+    #   params_schema = merge_jsonschema(
+    #       AirbyteDataSource.credentials_schema,
+    #       {"type": "object", "properties": {...}},
+    #   )
+    params_schema = {
+        "type": "object",
+        "properties": {
+            "normalization_mode": {
+                "type": "string",
+                "description": "Whether to normalize raw Airbyte tables. "
+                "`none` is no normalization, `basic` is Airbyte's basic normalization, "
+                "`custom` is a custom dbt transformation on the data.",
+                "enum": ["none", "basic", "custom"],
+                "default": ["basic"],
+            },
+            "normalization_git_branch": {
+                "type": "string",
+                "description": "Branch or commit hash to use for the normalization dbt project.",
+                "default": "master",
+            },
+        },
+    }
+
+    credentials_schema = {
+        "type": "object",
+        "properties": {
+            # This is a secret since the git URL might have a password in it.
+            "normalization_git_url": {
+                "type": "string",
+                "description": "For `custom` normalization, a URL to the Git repo "
+                "with the dbt project, for example,"
+                "`https://uname:pass_or_token@github.com/organisation/repository.git`.",
+            },
+        },
+    }
+
+    _internal_params = ["normalization_mode", "normalization_git_branch", "normalization_git_url"]
+
     def get_airbyte_config(self) -> AirbyteConfig:
-        return {**self.params, **self.credentials}
+        return {
+            k: v
+            for k, v in {**self.params, **self.credentials}.items()
+            if k not in self._internal_params
+        }
 
     def _sync(
         self,
@@ -97,7 +158,8 @@ class AirbyteDataSource(SyncableDataSource, ABC):
     def load(self, repository: "Repository", tables: Optional[TableInfo] = None) -> str:
         return self.sync(repository, image_hash=None, tables=tables, use_state=False)
 
-    def _make_postgres_config(self, engine: PostgresEngine, schema: str) -> AirbyteConfig:
+    @staticmethod
+    def _make_postgres_config(engine: PostgresEngine, schema: str) -> AirbyteConfig:
         return {
             "host": engine.conn_params["SG_ENGINE_HOST"],
             "port": int(engine.conn_params["SG_ENGINE_PORT"] or 5432),
@@ -179,36 +241,50 @@ class AirbyteDataSource(SyncableDataSource, ABC):
         # Set up a staging schema for the data
         # Delete the slashes or Airbyte will do it for us.
         staging_schema = "sg_tmp_" + repository.to_schema().replace("/", "_").replace("-", "_")
-        repository.object_engine.delete_schema(staging_schema)
-        repository.object_engine.create_schema(staging_schema)
-        repository.commit_engines()
-
         dst_config = self._make_postgres_config(repository.object_engine, staging_schema)
-
         client = get_docker_client()
         network_mode = detect_network_mode(client)
 
-        # Run the Airbyte source and receiver and pipe data between them, writing it
-        # out into a temporary schema.
+        with delete_schema_at_end(repository.object_engine, staging_schema):
+            repository.object_engine.delete_schema(staging_schema)
+            repository.object_engine.create_schema(staging_schema)
+            repository.commit_engines()
+            # Run the Airbyte source and receiver and pipe data between them, writing it
+            # out into a temporary schema.
 
-        logging.info("Running Airbyte EL process")
-        dest_files, new_state, sync_modes = self._run_airbyte_el(
-            client, network_mode, src_config, dst_config, configured_catalog, dst_catalog, state
-        )
+            logging.info("Running Airbyte EL process")
+            dest_files, new_state, sync_modes = self._run_airbyte_el(
+                client, network_mode, src_config, dst_config, configured_catalog, dst_catalog, state
+            )
 
-        # At this stage, Airbyte wrote out the raw tables into the staging schema: they have
-        # the form _airbyte_tmp_STREAM_NAME and schema (hash, raw_json, date). These raw tables
-        # are append-or-truncate only, so we append/replace them in the existing Splitgraph image
-        # at this stage.
+            # At this stage, Airbyte wrote out the raw tables into the staging schema: they have
+            # the form _airbyte_tmp_STREAM_NAME and schema (hash, raw_json, date). These raw tables
+            # are append-or-truncate only, so we append/replace them in the existing Splitgraph image
+            # at this stage.
 
-        logging.info("Storing raw tables as Splitgraph images")
-        raw_tables = _store_raw_airbyte_tables(
-            repository,
-            new_image_hash,
-            staging_schema,
-            sync_modes,
-            default_sync_mode="append_dedup" if use_state else "overwrite",
-        )
+            logging.info("Storing raw tables as Splitgraph images")
+            raw_tables = _store_raw_airbyte_tables(
+                repository,
+                new_image_hash,
+                staging_schema,
+                sync_modes,
+                default_sync_mode="append_dedup" if use_state else "overwrite",
+            )
+
+            # We can also store the state at this point, since only the raw tables depend on it.
+            store_ingestion_state(
+                repository,
+                new_image_hash,
+                current_state=state,
+                new_state=json.dumps(new_state) if new_state else "{}",
+            )
+            add_timestamp_tags(repository, new_image_hash)
+            repository.commit_engines()
+
+        normalization_mode = self.params.get("normalization_mode", "basic")
+
+        if normalization_mode == "none":
+            return new_image_hash
 
         # Run normalization
         # This converts the raw Airbyte tables (with JSON) into actual tables with fields.
@@ -217,32 +293,38 @@ class AirbyteDataSource(SyncableDataSource, ABC):
         # scan through them and build the actual ingested data.
 
         new_image = repository.images.by_hash(new_image_hash)
-        repository.object_engine.delete_schema(staging_schema)
-        repository.object_engine.create_schema(staging_schema)
-        new_image.lq_checkout(staging_schema, only_tables=raw_tables)
-        repository.commit_engines()
+        with delete_schema_at_end(repository.object_engine, staging_schema):
+            repository.object_engine.create_schema(staging_schema)
+            new_image.lq_checkout(staging_schema, only_tables=raw_tables)
+            repository.commit_engines()
+            logging.info("Running Airbyte T step (normalization)")
 
-        # Now run the normalization container
-        # This actually always recreates the normalized tables from scratch.
-        # https://github.com/airbytehq/airbyte/issues/4286
-        logging.info("Running Airbyte T step (normalization)")
-        with self._normalization_container(client, network_mode) as normalization_container:
-            add_files(normalization_container, dest_files)
-            normalization_container.start()
-            wait_not_failed(normalization_container, mirror_logs=True)
+            if normalization_mode == "basic":
+                # Run Airbyte's basic normalization container that autogenerates a dbt model from
+                # the catalog file.
+                # This actually always recreates the normalized tables from scratch.
+                # https://github.com/airbytehq/airbyte/issues/4286
+                logging.info("Using basic normalization")
+                with self._normalization_container(client, network_mode) as normalization_container:
+                    add_files(normalization_container, dest_files)
+                    normalization_container.start()
+                    wait_not_failed(normalization_container, mirror_logs=True)
+            else:
+                logging.info("Using a dbt project from Git")
 
-        logging.info("Storing processed Airbyte tables")
-        _store_processed_airbyte_tables(repository, new_image_hash, staging_schema)
+                try:
+                    git_url = self.credentials["normalization_git_url"]
+                except KeyError:
+                    raise ValueError("No normalization_git_url specified in plugin credentials!")
+                git_ref = self.params.get("normalization_git_branch", "master")
 
-        store_ingestion_state(
-            repository,
-            new_image_hash,
-            current_state=state,
-            new_state=json.dumps(new_state) if new_state else "{}",
-        )
-        add_timestamp_tags(repository, new_image_hash)
+                run_dbt_transformation_from_git(
+                    repository.object_engine, staging_schema, git_url, git_ref
+                )
 
-        repository.commit_engines()
+            logging.info("Storing normalized tables")
+            _store_processed_airbyte_tables(repository, new_image_hash, staging_schema)
+            repository.commit_engines()
 
         return new_image_hash
 
@@ -390,6 +472,7 @@ class AirbyteDataSource(SyncableDataSource, ABC):
             command=command,
             network_mode=network_mode,
             environment=self.docker_environment,
+            log_config=LogConfig(type="json-file", config={"max-size": "10m", "max-file": "3"}),
         )
         with remove_at_end(container):
             yield container
@@ -413,6 +496,7 @@ class AirbyteDataSource(SyncableDataSource, ABC):
             network_mode=network_mode,
             stdin_open=True,
             environment=self.docker_environment,
+            log_config=LogConfig(type="json-file", config={"max-size": "10m", "max-file": "3"}),
         )
         with remove_at_end(container):
             yield container
@@ -436,6 +520,7 @@ class AirbyteDataSource(SyncableDataSource, ABC):
             command=command,
             network_mode=network_mode,
             environment=self.docker_environment,
+            log_config=LogConfig(type="json-file", config={"max-size": "10m", "max-file": "3"}),
         )
 
         with remove_at_end(container):
