@@ -13,9 +13,10 @@ from splitgraph.config import CONFIG
 from splitgraph.core.image import Image
 from splitgraph.core.repository import Repository
 from splitgraph.core.types import TableSchema, TableColumn
+from splitgraph.engine.postgres.engine import get_change_key
 from splitgraph.exceptions import TableNotFoundError
 from splitgraph.ingestion.common import merge_tables
-from ._utils import _migrate_schema, log_exception, _make_changeset, rollback_at_end
+from .common import _migrate_schema, log_exception, _make_changeset, rollback_at_end
 
 
 def select_breadcrumb(stream_message, breadcrumb):
@@ -148,6 +149,7 @@ class DbSyncProxy(DbSync):
         self.image.repository.commit_engines()
 
     def _overwrite_existing_table(self, temp_table, table_name, schema_spec):
+        assert self.staging_table
         staging_table_schema, staging_table = self.staging_table
 
         # Merge the CSV data into the "materialized" table
@@ -190,8 +192,6 @@ class DbSyncProxy(DbSync):
             # Find PKs that have been upserted and deleted (make fake changeset)
             changeset = _make_changeset(
                 self.image.object_engine,
-                s,
-                old_table.table_name,
                 "pg_temp",
                 temp_table,
                 old_table.table_schema,
@@ -211,9 +211,20 @@ class DbSyncProxy(DbSync):
                 deleted,
             )
 
+            # Split the changeset according to the original fragment boundaries so that a single
+            # change spanning multiple fragments doesn't force materialization.
+            split_changesets = self.image.repository.objects.split_changeset_boundaries(
+                changeset, get_change_key(old_table.table_schema), old_table.objects
+            )
+            self.logger.info(
+                "Table %s: split changeset into %d fragment(s)",
+                get_table_name(self.stream_schema_message),
+                len(split_changesets),
+            )
+
             # Store the changeset as a new SG object
             object_ids = self.image.repository.objects._store_changesets(
-                old_table, [changeset], "pg_temp", table_name=temp_table
+                old_table, split_changesets, "pg_temp", table_name=temp_table
             )
 
         # Add the new object to the table.
@@ -221,7 +232,14 @@ class DbSyncProxy(DbSync):
         # the object to the table if it already exists)
         self.image.repository.objects.register_tables(
             self.image.repository,
-            [(self.image.image_hash, old_table.table_name, old_table.table_schema, object_ids,)],
+            [
+                (
+                    self.image.image_hash,
+                    old_table.table_name,
+                    old_table.table_schema,
+                    object_ids,
+                )
+            ],
         )
 
     def delete_rows(self, stream):
