@@ -1,8 +1,9 @@
 import base64
 import json
 import urllib.parse
-from typing import Dict, Optional, cast, Mapping, Any
+from typing import Dict, Optional, Any
 
+from splitgraph.core.types import TableInfo
 from splitgraph.hooks.data_source.fdw import ForeignDataWrapperDataSource
 from splitgraph.ingestion.common import build_commandline_help
 
@@ -38,7 +39,10 @@ class SnowflakeDataSource(ForeignDataWrapperDataSource):
                 "type": "string",
                 "description": "Account Locator, e.g. xy12345.us-east-2.aws. For more information, see https://docs.snowflake.com/en/user-guide/connecting.html",
             },
-            "private_key": {"type": "string", "description": "Private key in PEM format",},
+            "private_key": {
+                "type": "string",
+                "description": "Private key in PEM format",
+            },
         },
         "required": ["username", "account"],
         "oneOf": [{"required": ["password"]}, {"required": ["private_key"]}],
@@ -47,16 +51,14 @@ class SnowflakeDataSource(ForeignDataWrapperDataSource):
     params_schema = {
         "type": "object",
         "properties": {
-            "tables": {
-                "type": "object",
-                "additionalProperties": {
-                    "options": {"type": "object", "additionalProperties": {"type": "string"}},
-                },
-            },
             "database": {"type": "string", "description": "Snowflake database name"},
             "schema": {"type": "string", "description": "Snowflake schema"},
             "warehouse": {"type": "string", "description": "Warehouse name"},
             "role": {"type": "string", "description": "Role"},
+            "batch_size": {
+                "type": "integer",
+                "description": "Default fetch size for remote queries",
+            },
             "envvars": {
                 "type": "object",
                 "description": "Environment variables to set on the engine side",
@@ -65,6 +67,15 @@ class SnowflakeDataSource(ForeignDataWrapperDataSource):
         "required": ["database"],
     }
 
+    table_params_schema = {
+        "type": "object",
+        "properties": {
+            "subquery": {
+                "type": "string",
+                "description": "Subquery for this table to run on the server side",
+            }
+        },
+    }
     supports_mount = True
     supports_load = True
     supports_sync = False
@@ -128,8 +139,10 @@ EOF
     def get_description(cls) -> str:
         return "Schema, table or a subquery from a Snowflake database"
 
-    def get_table_options(self, table_name: str) -> Mapping[str, str]:
-        result = cast(Dict[str, str], super().get_table_options(table_name))
+    def get_table_options(
+        self, table_name: str, tables: Optional[TableInfo] = None
+    ) -> Dict[str, str]:
+        result = super().get_table_options(table_name, tables)
         result["tablename"] = result.get("tablename", table_name)
         return result
 
@@ -149,6 +162,9 @@ EOF
         if "envvars" in self.params:
             options["envvars"] = json.dumps(self.params["envvars"])
 
+        if "batch_size" in self.params:
+            options["batch_size"] = str(self.params["batch_size"])
+
         if "private_key" in self.credentials:
             options["connect_args"] = json.dumps(
                 {"private_key": _encode_private_key(self.credentials["private_key"])}
